package cn.com.qmth.examcloud.core.oe.student.api.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.support.examing.ExamQuestion;
import cn.com.qmth.examcloud.support.examing.ExamRecordQuestions;
import cn.com.qmth.examcloud.core.oe.student.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamStudentQuestionInfo;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamingSessionService;
import cn.com.qmth.examcloud.support.examing.ExamingHeartbeat;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import cn.com.qmth.examcloud.support.examing.ExamingStatus;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import javax.servlet.http.HttpServletRequest;

/**
 * @author  	chenken
 * @date    	2018年9月5日 下午4:57:05
 * @company 	QMTH
 * @description 考试作答记录controller
 */
@Api(tags = "考试过程中-试题相关接口")
@RestController
@RequestMapping("${app.api.oe.student}/examQuestion")
public class ExamQuestionController extends ControllerSupport {
	
	@Autowired
	private ExamRecordQuestionsService examRecordQuestionsService;
	
	@Autowired
    private ExamingSessionService examingSessionService;
	
    @Autowired
    private RedisClient redisClient;
	
	/**
	 * 将mongodb中的答过的题和redis中的题目列表合并返回给前端
	 * 返回给前端时注意将正确答案和得分置成null
	 * @return
	 */
	@ApiOperation(value = "考试过程中-获取试题列表")
	@GetMapping("/findExamQuestionList")
	public List<ExamQuestion> findExamQuestionList(){
		User user = getAccessUser();
		ExamingSession examSessionInfo = examingSessionService.getExamingSession(user.getUserId());
		if (examSessionInfo == null
				|| examSessionInfo.getExamingStatus().equals(ExamingStatus.INFORMAL)) {
			throw new StatusException("1001", "考试会话已过期，请重新开考");
		}
        
        String examingHeartbeatKey = RedisKeyHelper.getBuilder().examingHeartbeatKey(examSessionInfo.getExamRecordDataId());
        ExamingHeartbeat examingHeartbeat = redisClient.get(examingHeartbeatKey,ExamingHeartbeat.class);
        
		if (null != examingHeartbeat
				&& examingHeartbeat.getCost() >= examSessionInfo.getExamDuration()) {
			throw new StatusException("1001", "考试会话已过期，请重新开考");
		}
        
		ExamRecordQuestions qers = examRecordQuestionsService.getExamRecordQuestions(examSessionInfo.getExamRecordDataId());
		List<ExamQuestion> examQuestionList=qers.getExamQuestions();
		for(ExamQuestion examQuestion:examQuestionList){
			examQuestion.setCorrectAnswer(null);
			examQuestion.setStudentScore(null);
		}
		return examQuestionList;
	}

	/**
	 * 获取试题内容
	 * @param questionId
	 * @return
	 */
	@ApiOperation(value = "考试过程中-获取试题内容")
	@GetMapping("/getQuestionContent")
	public String getQuestionContent(@RequestParam String questionId){
		User user = getAccessUser();
		Check.isBlank(questionId, "questionId不能为空");
		return examRecordQuestionsService.getQuestionContent(user.getUserId(),questionId);
	}
	
	/**
	 * 考生作答
	 * @param examQuestionInfos
	 */
	@ApiOperation(value = "考试过程中-考生作答:更新试题作答信息(包括提交试题答案，更新是否标记)")
	@PostMapping("/submitQuestionAnswer")
	public void submitQuestionAnswer(@RequestBody List<ExamStudentQuestionInfo> examQuestionInfos,
									 HttpServletRequest request){
		if(LOGGER.isDebugEnabled()) {
			String strJosn=JsonUtil.toJson(examQuestionInfos);
			LOGGER.debug("ExamQuestionController--submitQuestionAnswer参数信息:"+strJosn);
		}
		User user = getAccessUser();
		if(examQuestionInfos!=null && examQuestionInfos.size()>0){
			for(ExamStudentQuestionInfo examStudentQuestionInfo:examQuestionInfos){
				if(examStudentQuestionInfo.getOrder() == null){
					throw new StatusException("2001", "illegal params");
				}
			}
			String referer = request.getHeader("REFERER");
			String agent = request.getHeader("USER-AGENT");
			examRecordQuestionsService.submitQuestionAnswer(user.getUserId(),examQuestionInfos,referer,agent);
		}
	}
}
