package cn.com.qmth.examcloud.core.oe.student.api.provider;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.student.api.ExamRecordDataCloudService;
import cn.com.qmth.examcloud.core.oe.student.api.bean.ExamFaceLivenessVerifyBean;
import cn.com.qmth.examcloud.core.oe.student.api.bean.FaceBiopsyBean;
import cn.com.qmth.examcloud.core.oe.student.api.bean.FaceBiopsyItemBean;
import cn.com.qmth.examcloud.core.oe.student.api.bean.FaceBiopsyItemStepBean;
import cn.com.qmth.examcloud.core.oe.student.api.request.*;
import cn.com.qmth.examcloud.core.oe.student.api.response.*;
import cn.com.qmth.examcloud.core.oe.student.dao.*;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.*;
import cn.com.qmth.examcloud.core.oe.student.service.ExamControlService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordPaperStructService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@Api(tags = "考试记录接口")
@RestController
@RequestMapping("${$rmp.cloud.oe.student}/examRecordData")
public class ExamRecordDataCloudServiceProvider extends ControllerSupport implements ExamRecordDataCloudService {


    /**
     *
     */
    private static final long serialVersionUID = 6142107111834463854L;
    @Autowired
    private ExamRecordDataService examRecordDataService;
    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;
    @Autowired
    private ExamFaceLivenessVerifyRepo examFaceLivenessVerifyRepo;
    @Autowired
    private FaceBiopsyRepo faceBiopsyRepo;
    @Autowired
    private FaceBiopsyItemRepo faceBiopsyItemRepo;
    @Autowired
    private FaceBiopsyItemStepRepo faceBiopsyItemStepRepo;
    @Autowired
    private ExamRecordPaperStructService examRecordPaperStructService;
    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;
    @Autowired
    private ExamControlService examControlService;

    @Override
    @ApiOperation(value = "批量获取未同步的考试记录ID")
    @PostMapping("/getExamRecordDataIds")
    public GetExamRecordDataIdsResp getExamRecordDataIds(@RequestBody GetExamRecordDataIdsReq req) {
        GetExamRecordDataIdsResp res = new GetExamRecordDataIdsResp();
        List<Long> ids = examRecordDataService.getExamRecordDataIds(req);
        res.setExamRecordDataIds(ids);
        return res;
    }

    @ApiOperation(value = "批量修改考试记录batchNum")
    @PostMapping("/updateBatchNum")
    @Override
    public UpdateExamRecordDataBatchNumResp updateExamRecordDataBatchNum(@RequestBody UpdateExamRecordDataBatchNumReq req) {
        examRecordDataService.updateExamRecordDataBatchNum(req);
        UpdateExamRecordDataBatchNumResp res = new UpdateExamRecordDataBatchNumResp();
        return res;
    }

    /**
     * 计算活体检测结果
     *
     * @param req
     * @return
     */
    @ApiOperation(value = "计算活体检测结果")
    @PostMapping("/calcFaceBiopsyResult")
    @Override
    public CalcFaceBiopsyResultResp calcFaceBiopsyResult(@RequestBody CalcFaceBiopsyResultReq req) {
        return examRecordDataService.calcFaceBiopsyResult(req);
    }

    /**
     * 计算考试分数
     *
     * @param req
     * @return
     */
    @ApiOperation(value = "计算考试分数")
    @PostMapping("/calcExamScore")
    @Override
    public CalcExamScoreResp calcExamScore(@RequestBody CalcExamScoreReq req) {
        return examRecordDataService.calcExamScore(req);
    }

    /**
     * 获取旧活体检测结果
     *
     * @param req
     * @return
     */
    @ApiOperation(value = "获取旧活体检测结果")
    @PostMapping("/getExamFaceLivenessVerifies")
    @Override
    public GetExamFaceLivenessVerifiesResp getExamFaceLivenessVerifies(@RequestBody GetExamFaceLivenessVerifiesReq req) {
        List<ExamFaceLivenessVerifyEntity> entityList = examFaceLivenessVerifyRepo.findByExamRecordDataId(req.getExamRecordDataId());

        List<ExamFaceLivenessVerifyBean> examFaceLivenessVerifyList = new ArrayList<>();

        for (ExamFaceLivenessVerifyEntity entity : entityList) {
            ExamFaceLivenessVerifyBean bean = new ExamFaceLivenessVerifyBean();
            bean.setBizId(entity.getBizId());
            bean.setIsError(entity.getIsError());
            bean.setErrorMsg(entity.getErrorMsg());
            bean.setExamRecordDataId(entity.getExamRecordDataId());
            bean.setId(entity.getId());
            bean.setOperateNum(entity.getOperateNum());
            bean.setResultJson(entity.getResultJson());
            bean.setStartTime(entity.getStartTime());
            bean.setUsedTime(entity.getUsedTime());
            if (null != entity.getVerifyResult()) {
                bean.setVerifyResult(entity.getVerifyResult().name());
            }

            examFaceLivenessVerifyList.add(bean);

        }

        GetExamFaceLivenessVerifiesResp resp = new GetExamFaceLivenessVerifiesResp();
        resp.setExamFaceLivenessVerifis(examFaceLivenessVerifyList);

        return resp;
    }

    /**
     * 获取新活检
     *
     * @param req
     * @return
     */
    @ApiOperation(value = "获取新活检")
    @PostMapping("/getFaceBiopsy")
    @Override
    public GetFaceBiopsyResp getFaceBiopsy(@RequestBody GetFaceBiopsyReq req) {

        FaceBiopsyEntity entity = faceBiopsyRepo.findByExamRecordDataId(req.getExamRecordDataId());

        if (null == entity) {
            return new GetFaceBiopsyResp();
        }

        List<FaceBiopsyItemEntity> itemEntityList = faceBiopsyItemRepo.findByFaceBiopsyIdOrderByIdAsc(entity.getId());

        List<FaceBiopsyItemBean> itemBeanList = new ArrayList<>();

        for (FaceBiopsyItemEntity itemEntity : itemEntityList) {
            FaceBiopsyItemBean itemBean = new FaceBiopsyItemBean();

            itemBean.setCompleted(itemEntity.getCompleted());
            itemBean.setErrorMsg(itemEntity.getErrorMsg());
            itemBean.setExamRecordDataId(itemEntity.getExamRecordDataId());
            itemBean.setFaceBiopsyId(itemEntity.getFaceBiopsyId());
            itemBean.setFaceBiopsyItemSteps(null);
            itemBean.setFaceBiopsyType(itemEntity.getFaceBiopsyType().name());
            itemBean.setInFreezeTime(itemEntity.getInFreezeTime());
            itemBean.setResult(itemEntity.getResult());

            List<FaceBiopsyItemStepBean> itemStepBeanList = new ArrayList<>();

            List<FaceBiopsyItemStepEntity> itemStepEntityList = faceBiopsyItemStepRepo.findByFaceBiopsyItemId(itemEntity.getId());

            for (FaceBiopsyItemStepEntity itemStepEntity : itemStepEntityList) {

                FaceBiopsyItemStepBean itemStepBean = new FaceBiopsyItemStepBean();

                if (null != itemStepEntity.getAction()) {
                    itemStepBean.setAction(itemStepEntity.getAction().name());
                }

                itemStepBean.setActionStay(itemStepEntity.getActionStay());
                itemStepBean.setErrorMsg(itemStepEntity.getErrorMsg());
                itemStepBean.setExamRecordDataId(itemStepEntity.getExamRecordDataId());
                itemStepBean.setFaceBiopsyItemId(itemStepEntity.getFaceBiopsyItemId());
                itemStepBean.setResourceRelativePath(itemStepEntity.getResourceRelativePath());

                if (null != itemStepEntity.getResourceType()) {
                    itemStepBean.setResourceType(itemStepEntity.getResourceType().name());
                }

                itemStepBean.setResult(itemStepEntity.getResult());

                itemStepBeanList.add(itemStepBean);
            }

            itemBean.setFaceBiopsyItemSteps(itemStepBeanList);

            itemBeanList.add(itemBean);
        }

        FaceBiopsyBean bean = new FaceBiopsyBean();
        bean.setErrorMsg(entity.getErrorMsg());
        bean.setExamRecordDataId(req.getExamRecordDataId());
        bean.setResult(entity.getResult());
        bean.setRootOrgId(entity.getRootOrgId());
        bean.setVerifiedTimes(entity.getVerifiedTimes());

        bean.setFaceBiopsyItems(itemBeanList);

        GetFaceBiopsyResp resp = new GetFaceBiopsyResp();
        resp.setFaceBiopsyBean(bean);

        return resp;

    }

    /**
     * 获取考试作答记录
     *
     * @param req
     * @return
     */
    @ApiOperation(value = "获取考试作答记录")
    @PostMapping("/getExamRecordQuestions")
    @Override
    public GetExamRecordQuestionsResp getExamRecordQuestions(@RequestBody GetExamRecordQuestionsReq req) {
        return examRecordQuestionsService.getExamRecordQuestions(req);
    }

    /**
     * 获取试卷结构
     *
     * @param req
     * @return
     */
    @ApiOperation(value = "获取试卷结构")
    @PostMapping("/getExamRecordPaperStruct")
    @Override
    public GetExamRecordPaperStructResp getExamRecordPaperStruct(@RequestBody GetExamRecordPaperStructReq req) {

        return examRecordPaperStructService.getExamRecordPaperStruct(req);
    }

    /**
     * 交卷
     *
     * @param req
     * @return
     */
    @ApiOperation(value = "交卷")
    @PostMapping("/handInExam")
    @Override
    public HandInExamResp handInExam(@RequestBody HandInExamReq req) {
        examControlService.handInExam(req.getExamRecordDataId(), req.getHandInExamType(),req.getIp());

        return new HandInExamResp();
    }

    @ApiOperation(value = "修改考试记录状态")
    @PostMapping("/updateExamRecordStatus")
    @Override
    public UpdateExamRecordStatusResp updateExamRecordStatus(@RequestBody UpdateExamRecordStatusReq req) {
        examRecordDataService.updateExamRecordStatus(req);
        UpdateExamRecordStatusResp res = new UpdateExamRecordStatusResp();
        return res;
    }

    /**
     * 获取考试记录数量
     *
     * @param req
     * @return
     */
    @ApiOperation(value = "获取考试记录数量")
    @PostMapping("/getExamRecordNum")
    @Override
    public GetExamRecordNumResp getExamRecordNum(@RequestBody GetExamRecordNumReq req) {
        ExamRecordDataEntity query = new ExamRecordDataEntity();
        query.setExamId(req.getExamId());
        Example<ExamRecordDataEntity> queryExample = Example.of(query);
        long num = examRecordDataRepo.count(queryExample);

        GetExamRecordNumResp resp = new GetExamRecordNumResp();
        resp.setNum(num);

        return resp;
    }

    @ApiOperation(value = "修改部分考试记录")
    @PostMapping("/updatePartialExamRecord")
    @Override
    public UpdatePartialExamRecordResp updatePartialExamRecord(@RequestBody UpdatePartialExamRecordReq req) {
        examRecordDataService.updatePartialExamRecord(req);
        UpdatePartialExamRecordResp res = new UpdatePartialExamRecordResp();
        return res;
    }

	@ApiOperation(value = "检查试卷是否已使用")
    @PostMapping("/checkPaperInExam")
	@Override
	public CheckPaperInExamResp checkPaperInExam(@RequestBody CheckPaperInExamReq req) {
		if (org.apache.commons.lang3.StringUtils.isBlank(req.getBasePaperId())) {
            throw new StatusException("500", "basePaperId不允许为空");
        }
		return examRecordDataService.checkPaperInExam(req);
	}
}
