package cn.com.qmth.examcloud.core.oe.student.dao.entity;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Index;
import javax.persistence.Table;

import cn.com.qmth.examcloud.support.enums.IsSuccess;
import cn.com.qmth.examcloud.support.enums.SyncStatus;
import org.hibernate.annotations.DynamicInsert;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.support.enums.HandInExamType;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.web.jpa.JpaEntity;
import org.hibernate.annotations.DynamicUpdate;

/**
 * @author chenken
 * @date 2018年8月13日 上午11:41:34
 * @company QMTH
 * @description 考试记录数据表, 与ec_oe_exam_record一对一关系
 */
@Entity
@Table(name = "ec_oes_exam_record_data", indexes = {
        @Index(name = "IDX_E_O_E_R_D_001", columnList = "examStudentId"),
        @Index(name = "IDX_E_O_E_R_D_002", columnList = "studentId"),
        @Index(name = "IDX_E_O_E_R_D_003", columnList = "examId"),
        @Index(name = "IDX_E_O_E_R_D_004", columnList = "courseId"),
        @Index(name = "IDX_E_O_E_R_D_005", columnList = "batchNum"),
        @Index(name = "IDX_E_O_E_R_D_006", columnList = "creationTime"),
        @Index(name = "IDX_E_O_E_R_D_007", columnList = "basePaperId"),
})
@DynamicInsert
@DynamicUpdate
public class ExamRecordDataEntity extends JpaEntity {
    /**
     *
     */
    private static final long serialVersionUID = -242327915801750970L;
    /**
     * 主键
     */
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;
    /**
     * 考试ID
     */
    private Long examId;
    /**
     * 考试类型
     */
    @Column(length = 20)
    @Enumerated(EnumType.STRING)
    private ExamType examType;
    /**
     * 考生ID
     */
    private Long examStudentId;
    /**
     * 学生ID
     */
    private Long studentId;
    /**
     * 课程ID
     */
    private Long courseId;

    /**
     * 学习中心ID
     */
    private Long orgId;
    /**
     * 顶级机构ID
     */
    private Long rootOrgId;
    /**
     * 基础试卷ID
     */
    private String basePaperId;

    /**
     * 试卷类型
     */
    private String paperType;

    /**
     * 考试开始时间
     */
    private Date startTime;
    /**
     * 考试结束时间
     */
    private Date endTime;

    /**
     * 考试被清理时间
     */
    private Date cleanTime;

    /**
     * 考试时长
     */
    private Long usedExamTime;
    /**
     * 是否断点续考
     */
    private Boolean isContinued;
    /**
     * 断点续考次数
     */
    private Integer continuedCount;
    /**
     * 是否达到最大断点限制
     */
    private Boolean isExceed;
    /**
     * 抓拍比对成功次数
     */
    private Integer faceSuccessCount;
    /**
     * 抓拍比对失败次数
     */
    private Integer faceFailedCount;
    /**
     * 抓拍存在陌生人的次数
     */
    private Integer faceStrangerCount;
    /**
     * 抓拍比对总次数
     */
    private Integer faceTotalCount;

    /**
     * 考试记录状态
     */
    @Column(length = 20)
    @Enumerated(EnumType.STRING)
    private ExamRecordStatus examRecordStatus;

    /**
     * 交卷类型
     */
    @Column(length = 20)
    @Enumerated(EnumType.STRING)
    private HandInExamType handInExamType;

    /**
     *
     * 活体检测结果
     */
    @Column(length = 20)
    @Enumerated(EnumType.STRING)
    private IsSuccess faceVerifyResult;

    /**
     * 是否异常数据
     */
    private Boolean isWarn;
    /**
     * 是否被审核过
     */
    private Boolean isAudit;
    /**
     * 是否违纪
     */
    @Column(name = "is_illegality")
    private Boolean isIllegality;

    /**
     * 总分
     */
    private Double totalScore;

    /**
     * 客观题得分总分
     */
    private Double objectiveScore;

    /**
     * 客观题答对的比率
     * (客观题答对的题数/客观题总题数)*100  取2位小数
     */
    private Double objectiveAccuracy;

    /**
     * 主观题得分总分
     */
    private Double subjectiveScore;

    /**
     * 答题正确率
     */
    private Double succPercent;

    /**
     * 抓拍比对成功比率
     */
    private Double faceSuccessPercent;

    /**
     * 百度人脸活体检测通过率
     */
    private Double baiduFaceLivenessSuccessPercent;

    /**
     * 数据同步状态
     */
    @Column(length = 20)
    @Enumerated(EnumType.STRING)
    private SyncStatus syncStatus;

    /**
     * 批次号，自动服务遍历数据用
     */
    private Long batchNum;

    /**
     * 试卷题目数量（校验提交答案的order）
     */
    private Integer questionCount;

    /**
     * 是否是全客观题卷  1:是   0:否
     */
    private Boolean isAllObjectivePaper;

    /**
     * 场次id
     */
    private Long examStageId;

    /**
     * 场次号
     */
    private Integer examStageOrder;

    /**
     * 学生最后活动时间
     */
    private Date lastActiveTime;

    /**
     * 断点续考时间
     */
    private Date continuedTime;

    /**
     * 进入考试时间
     */
    private Date enterExamTime;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public ExamType getExamType() {
        return examType;
    }

    public void setExamType(ExamType examType) {
        this.examType = examType;
    }

    public Long getExamStudentId() {
        return examStudentId;
    }

    public void setExamStudentId(Long examStudentId) {
        this.examStudentId = examStudentId;
    }

    public Long getStudentId() {
        return studentId;
    }

    public void setStudentId(Long studentId) {
        this.studentId = studentId;
    }

    public Long getCourseId() {
        return courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public Long getOrgId() {
        return orgId;
    }

    public void setOrgId(Long orgId) {
        this.orgId = orgId;
    }

    public Long getRootOrgId() {
        return rootOrgId;
    }

    public void setRootOrgId(Long rootOrgId) {
        this.rootOrgId = rootOrgId;
    }

    public String getBasePaperId() {
        return basePaperId;
    }

    public void setBasePaperId(String basePaperId) {
        this.basePaperId = basePaperId;
    }

    public String getPaperType() {
        return paperType;
    }

    public void setPaperType(String paperType) {
        this.paperType = paperType;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    public Long getUsedExamTime() {
        return usedExamTime;
    }

    public void setUsedExamTime(Long usedExamTime) {
        this.usedExamTime = usedExamTime;
    }

    public Boolean getIsContinued() {
        return isContinued;
    }

    public void setIsContinued(Boolean isContinued) {
        this.isContinued = isContinued;
    }

    public Integer getContinuedCount() {
        return continuedCount;
    }

    public void setContinuedCount(Integer continuedCount) {
        this.continuedCount = continuedCount;
    }

    public Integer getFaceSuccessCount() {
        return faceSuccessCount;
    }

    public void setFaceSuccessCount(Integer faceSuccessCount) {
        this.faceSuccessCount = faceSuccessCount;
    }

    public Integer getFaceFailedCount() {
        return faceFailedCount;
    }

    public void setFaceFailedCount(Integer faceFailedCount) {
        this.faceFailedCount = faceFailedCount;
    }

    public Integer getFaceStrangerCount() {
        return faceStrangerCount;
    }

    public void setFaceStrangerCount(Integer faceStrangerCount) {
        this.faceStrangerCount = faceStrangerCount;
    }

    public Integer getFaceTotalCount() {
        return faceTotalCount;
    }

    public void setFaceTotalCount(Integer faceTotalCount) {
        this.faceTotalCount = faceTotalCount;
    }

    public Boolean getIsExceed() {
        return isExceed;
    }

    public void setIsExceed(Boolean isExceed) {
        this.isExceed = isExceed;
    }

    public ExamRecordStatus getExamRecordStatus() {
        return examRecordStatus;
    }

    public void setExamRecordStatus(ExamRecordStatus examRecordStatus) {
        this.examRecordStatus = examRecordStatus;
    }

    public HandInExamType getHandInExamType() {
        return handInExamType;
    }

    public void setHandInExamType(HandInExamType handInExamType) {
        this.handInExamType = handInExamType;
    }

    public IsSuccess getFaceVerifyResult() {
        return faceVerifyResult;
    }

    public void setFaceVerifyResult(IsSuccess faceVerifyResult) {
        this.faceVerifyResult = faceVerifyResult;
    }

    public Date getCleanTime() {
        return cleanTime;
    }

    public void setCleanTime(Date cleanTime) {
        this.cleanTime = cleanTime;
    }

    public Double getTotalScore() {
        return totalScore;
    }

    public void setTotalScore(Double totalScore) {
        this.totalScore = totalScore;
    }

    public Double getObjectiveScore() {
        return objectiveScore;
    }

    public void setObjectiveScore(Double objectiveScore) {
        this.objectiveScore = objectiveScore;
    }

    public Double getObjectiveAccuracy() {
        return objectiveAccuracy;
    }

    public void setObjectiveAccuracy(Double objectiveAccuracy) {
        this.objectiveAccuracy = objectiveAccuracy;
    }

    public Double getSubjectiveScore() {
        return subjectiveScore;
    }

    public void setSubjectiveScore(Double subjectiveScore) {
        this.subjectiveScore = subjectiveScore;
    }

    public Double getSuccPercent() {
        return succPercent;
    }

    public void setSuccPercent(Double succPercent) {
        this.succPercent = succPercent;
    }

    public Boolean getIsWarn() {
        return isWarn;
    }

    public void setIsWarn(Boolean warn) {
        isWarn = warn;
    }

    public Boolean getIsAudit() {
        return isAudit;
    }

    public void setIsAudit(Boolean audit) {
        isAudit = audit;
    }

    public Boolean getIsIllegality() {
        return isIllegality;
    }

    public void setIsIllegality(Boolean illegality) {
        isIllegality = illegality;
    }

    public SyncStatus getSyncStatus() {
        return syncStatus;
    }


    public void setSyncStatus(SyncStatus syncStatus) {
        this.syncStatus = syncStatus;
    }


    public Long getBatchNum() {
        return batchNum;
    }


    public void setBatchNum(Long batchNum) {
        this.batchNum = batchNum;
    }

    public Double getFaceSuccessPercent() {
        return faceSuccessPercent;
    }

    public void setFaceSuccessPercent(Double faceSuccessPercent) {
        this.faceSuccessPercent = faceSuccessPercent;
    }

    public Double getBaiduFaceLivenessSuccessPercent() {
        return baiduFaceLivenessSuccessPercent;
    }

    public void setBaiduFaceLivenessSuccessPercent(Double baiduFaceLivenessSuccessPercent) {
        this.baiduFaceLivenessSuccessPercent = baiduFaceLivenessSuccessPercent;
    }


    public Integer getQuestionCount() {
        return questionCount;
    }


    public void setQuestionCount(Integer questionCount) {
        this.questionCount = questionCount;
    }


    public Boolean getIsAllObjectivePaper() {
        return isAllObjectivePaper;
    }


    public void setIsAllObjectivePaper(Boolean isAllObjectivePaper) {
        this.isAllObjectivePaper = isAllObjectivePaper;
    }

    public Long getExamStageId() {
        return examStageId;
    }

    public void setExamStageId(Long examStageId) {
        this.examStageId = examStageId;
    }

    public Integer getExamStageOrder() {
        return examStageOrder;
    }

    public void setExamStageOrder(Integer examStageOrder) {
        this.examStageOrder = examStageOrder;
    }

    public Date getLastActiveTime() {
        return lastActiveTime;
    }

    public void setLastActiveTime(Date lastActiveTime) {
        this.lastActiveTime = lastActiveTime;
    }

    public Date getContinuedTime() {
        return continuedTime;
    }

    public void setContinuedTime(Date continuedTime) {
        this.continuedTime = continuedTime;
    }

    public Date getEnterExamTime() {
        return enterExamTime;
    }

    public void setEnterExamTime(Date enterExamTime) {
        this.enterExamTime = enterExamTime;
    }
}
