package cn.com.qmth.examcloud.core.oe.student.service;


import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.core.oe.student.bean.*;
import cn.com.qmth.examcloud.support.enums.HandInExamType;

import javax.validation.constraints.Null;

/**
 * @author chenken
 * @date 2018年8月13日 下午2:09:38
 * @company QMTH
 * @description 在线考试控制服务接口
 */
public interface ExamControlService {

    /**
     * 开始考试
     *
     * @param examStudentId
     * @param user
     */
    StartExamInfo startExam(Long examStudentId, User user, String ip);

    /**
     * 开始答题
     *
     * @param examRecordDataId
     */
    StartAnswerInfo startAnswer(Long examRecordDataId);

    /**
     * 交卷
     *
     * @param examRecordDataId 考试记录id
     * @param handInExamType   交卷类型
     * @param ip 请求ip，可以为空
     */
    void handInExam(Long examRecordDataId, HandInExamType handInExamType,@Null String ip);

    /**
     * 断点续考:检查正在进行中的考试
     *
     * @param studentId
     */
    CheckExamInProgressInfo checkExamInProgress(Long studentId,String ip);

    /**
     * 考试心跳
     *
     * @param
     */
    long examHeartbeat(User user, String ip);

    /**
     * 获取考试结束后的相关信息
     *
     * @param examRecordDataId
     * @return
     */
    EndExamInfo getEndExamInfo(Long examRecordDataId);

    CheckQrCodeInfo checkQrCode(String qrCode);

    /**
     * 发送消息到websocket
     *
     * @param examRecordDataId 考试记录id
     * @param order            题序号
     * @param fileUrl          文件路径
     * @param transferFileType 传输文件类型
     * @throws Exception
     */
    void sendFileAnswerToWebSocket(Long examRecordDataId, Integer order,
                                   String fileUrl, String transferFileType, Long userId, Long rootOrgId) throws Exception;

    /**
     * 通过websocket发送二维码扫描信息
     *
     * @param examRecordDataId
     * @param clientId
     * @throws Exception
     */
    void sendScanQrCodeToWebSocket(String clientId, Long examRecordDataId, Integer order,
                                   Long userId, Long rootOrgId) throws Exception;

    UpyunSignatureInfo getUpyunSignature(GetYunSignatureReq req);

    /**
     * 获取二维码
     * @param req
     * @param user
     * @return
     */
    String getQrCode(GetQrCodeReq req, User user);

    /**
     * 记录切换屏幕次数
     *
     * @param examRecordDataId
     */
    void switchScreen(Long examRecordDataId);
}
