package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.core.oe.student.base.utils.CommonUtil;
import cn.com.qmth.examcloud.core.oe.student.base.utils.FileDisposeUtil;
import cn.com.qmth.examcloud.core.oe.student.base.utils.HttpPoolUtil;
import cn.com.qmth.examcloud.core.oe.student.bean.GetFaceVerifyTokenInfo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamFaceLivenessVerifyRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamFaceLivenessVerifyEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.enums.FaceVerifyResult;
import cn.com.qmth.examcloud.core.oe.student.service.ExamControlService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamFaceLivenessVerifyService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamingSessionService;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.StudentCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import cn.com.qmth.examcloud.support.enums.HandInExamType;
import cn.com.qmth.examcloud.support.enums.IsSuccess;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.support.helper.FaceBiopsyHelper;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


/**
 * @author chenken
 * @date 2018/8/15 15:57
 * @company QMTH
 * @description 活体检测服务实现
 */
@Service("examFaceLivenessVerifyService")
public class ExamFaceLivenessVerifyServiceImpl implements ExamFaceLivenessVerifyService {

    private static final Logger log = LoggerFactory.getLogger(ExamFaceLivenessVerifyServiceImpl.class);

    @Autowired
    private ExamFaceLivenessVerifyRepo examFaceLivenessVerifyRepo;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private ExamingSessionService examingSessionService;

    @Autowired
    private ExamControlService examControlService;
    @Autowired
    SystemProperties systemProperties;

    /**
     * 第二次人脸检测时间范围
     */
    private static final int secondFaceCheckMinute = 4;

    @Override
    public ExamFaceLivenessVerifyEntity saveFaceVerify(Long examRecordDataId) {
        ExamFaceLivenessVerifyEntity faceVerify = new ExamFaceLivenessVerifyEntity();
        faceVerify.setExamRecordDataId(examRecordDataId);
        faceVerify.setStartTime(new Date());
        faceVerify.setIsError(false);
        faceVerify.setOperateNum(1);
        return examFaceLivenessVerifyRepo.save(faceVerify);
    }

    @Override
    public List<ExamFaceLivenessVerifyEntity> listFaceVerifyByExamRecordId(Long examRecordDataId) {
        if (examRecordDataId == null) {
            return new ArrayList<ExamFaceLivenessVerifyEntity>();
        }
        return examFaceLivenessVerifyRepo.findByExamRecordDataIdOrderById(examRecordDataId);
    }

    @Override
    public ExamFaceLivenessVerifyEntity faceIdNotify(String faceIdResultJson) {
        JSONObject faceIdResultJsonObject;
        String bizNo;

        ExamFaceLivenessVerifyEntity faceVerify = new ExamFaceLivenessVerifyEntity();
        try {
            faceIdResultJsonObject = new JSONObject(faceIdResultJson);
            bizNo = faceIdResultJsonObject.getString("biz_no");
            Long faceVerifyId;
            if (bizNo.indexOf("_") == -1) {
                faceVerifyId = Long.parseLong(bizNo + "");
            } else {
                faceVerifyId = Long.parseLong(bizNo.substring(0, bizNo.indexOf("_")));
            }

            faceVerify = findFaceVerifyById(faceVerifyId);
            if (faceIdResultJsonObject.has("verify_result")
                    && faceIdResultJsonObject.get("verify_result") != null
                    && !"null".equals(faceIdResultJsonObject.get("verify_result") + "")) {
                JSONObject verifyResultJson = faceIdResultJsonObject.getJSONObject("verify_result");
                JSONObject result_ref1 = verifyResultJson.getJSONObject("result_ref1");
                double confidence = result_ref1.getDouble("confidence");
                //人脸相似度
                if (confidence > 50D) {
                    JSONObject livenessResultJson = faceIdResultJsonObject.getJSONObject("liveness_result");
                    if (livenessResultJson.has("result") && "success".equals(livenessResultJson.getString("result"))) {
                        /*JSONObject faceGenuineness = verifyResultJson.getJSONObject("face_genuineness");
                        //如果开启了翻拍检测
                        if(faceGenuineness.has("screen_replay_threshold") && faceGenuineness.has("screen_replay_confidence")){
                            double replayThreshold = faceGenuineness.getDouble("screen_replay_threshold");
                            double replayConfidence = faceGenuineness.getDouble("screen_replay_confidence");
                            //如果大于阈值,表明为翻拍
                            if (replayConfidence >= replayThreshold) {
                                faceVerify.setVerifyResult(FaceVerifyResult.SCREEN_REPLAY);
                            } else {
                                faceVerify.setVerifyResult(FaceVerifyResult.VERIFY_SUCCESS);
                            }
                        } else {
                            faceVerify.setVerifyResult(FaceVerifyResult.VERIFY_SUCCESS);
                        }*/
                        faceVerify.setVerifyResult(FaceVerifyResult.VERIFY_SUCCESS);
                    }
                } else {
                    faceVerify.setVerifyResult(FaceVerifyResult.NOT_ONESELF);
                }
            } else {
                faceVerify.setVerifyResult(FaceVerifyResult.VERIFY_FAILED);
            }
            faceVerify.setResultJson(faceIdResultJsonObject.toString());
            faceVerify.setBizId(faceIdResultJsonObject.getString("biz_id"));
        } catch (Exception e) {
            log.error("faceIdNotify error", e);
            faceVerify.setVerifyResult(FaceVerifyResult.VERIFY_FAILED);
        }
        long usedTime = System.currentTimeMillis() - faceVerify.getStartTime().getTime();
        faceVerify.setUsedTime(usedTime);
        return examFaceLivenessVerifyRepo.save(faceVerify);
    }

    @Override
    public GetFaceVerifyTokenInfo getFaceVerifyToken(Long studentId, String bizNo) {
        GetFaceVerifyTokenInfo getFaceVerifyTokenInfo = new GetFaceVerifyTokenInfo();
        CloseableHttpResponse httpResponse = null;
        CloseableHttpClient httpClient = null;
        try {
            httpClient = HttpPoolUtil.getHttpClient();
            HttpPost httpPost = new HttpPost(PropertyHolder.getString("app.faceid.get_token_url"));
            File basePhotoFile = getStudentBasePhotoFile(studentId);
            MultipartEntityBuilder multipartEntityBuilder = getMultipartEntityBuilder(bizNo, basePhotoFile);
            HttpEntity httpEntity = multipartEntityBuilder.build();
            httpPost.setEntity(httpEntity);
            httpResponse = httpClient.execute(httpPost);
            HttpEntity responseEntity = httpResponse.getEntity();
            BufferedReader reader = new BufferedReader(new InputStreamReader(responseEntity.getContent()));
            StringBuffer buffer = new StringBuffer();
            String str = "";
            while (StringUtils.isNoneBlank((str = reader.readLine()))) {
                buffer.append(str);
            }
            String result = buffer.toString();
            basePhotoFile.delete();
            JSONObject jsonObject = new JSONObject(result);
            if (jsonObject.has("error_message")) {
                getFaceVerifyTokenInfo.setSuccess(false);
                getFaceVerifyTokenInfo.setErrorMsg(jsonObject.getString("error_message"));
                log.error("getFaceVerifyToken error", result);
            } else if (jsonObject.has("error")) {
                getFaceVerifyTokenInfo.setSuccess(false);
                getFaceVerifyTokenInfo.setErrorMsg(jsonObject.getString("error"));
                log.error("getFaceVerifyToken error", result);
            } else if (jsonObject.has("token")) {
                getFaceVerifyTokenInfo.setSuccess(true);
                getFaceVerifyTokenInfo.setFaceLivenessToken(jsonObject.getString("token"));
            }
            httpPost.abort();
        } catch (Exception e) {
            log.error("getFaceVerifyToken error", e);
            getFaceVerifyTokenInfo.setSuccess(false);
            getFaceVerifyTokenInfo.setErrorMsg(e.getMessage());
        } finally {
            if (httpResponse != null) {
                try {
                    httpResponse.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return getFaceVerifyTokenInfo;
    }

    @Override
    public ExamFaceLivenessVerifyEntity findFaceVerifyById(Long id) {
        if (id == null) {
            return null;
        }
        return GlobalHelper.getEntity(examFaceLivenessVerifyRepo, id, ExamFaceLivenessVerifyEntity.class);
    }

    @Override
    public void faceTestTimeOut(Long examRecordDataId) {
        List<ExamFaceLivenessVerifyEntity> faceVerifies = examFaceLivenessVerifyRepo.findByExamRecordDataIdOrderById(examRecordDataId);
        ExamFaceLivenessVerifyEntity faceVerify = faceVerifies.get(faceVerifies.size() - 1);
        if (faceVerify.getVerifyResult() == null) {
            faceVerify.setVerifyResult(FaceVerifyResult.TIME_OUT);
            examFaceLivenessVerifyRepo.save(faceVerify);
        }
    }

    @Override
    public void faceTestEndHandle(Long examRecordDataId, Long studentId, String result) {
        //保存活体检测的结果
        ExamRecordData examRecordData = examRecordDataService.getExamRecordDataCache(examRecordDataId);
        examRecordData.setFaceVerifyResult(IsSuccess.strToEnum(result) == IsSuccess.FAILED ? IsSuccess.FAILED : IsSuccess.SUCCESS);

        //活检失败，则认为违纪
        if (IsSuccess.strToEnum(result) == IsSuccess.FAILED) {
            examRecordData.setIsIllegality(true);
        }

        examRecordDataService.saveExamRecordDataCache(examRecordDataId, examRecordData);

        //如果活体检失败，需要清除会话并自动交卷
        if (IsSuccess.strToEnum(result) == IsSuccess.FAILED) {
            examControlService.handInExam(examRecordDataId, HandInExamType.AUTO,null);
        }
    }

    @Override
    public ExamFaceLivenessVerifyEntity saveFaceVerifyByExamRecordDataId(Long examRecordDataId) {
        ExamFaceLivenessVerifyEntity examFaceVerifyEntity = examFaceLivenessVerifyRepo.findErrorFaceVerifyByExamRecordDataId(examRecordDataId);
        if (examFaceVerifyEntity != null) {
            examFaceVerifyEntity.setStartTime(new Date());
            examFaceVerifyEntity.setIsError(false);
            examFaceVerifyEntity.setErrorMsg(null);
            examFaceVerifyEntity.setOperateNum(examFaceVerifyEntity.getOperateNum().intValue() + 1);
            return examFaceLivenessVerifyRepo.save(examFaceVerifyEntity);
        } else {
            examFaceVerifyEntity = saveFaceVerify(examRecordDataId);
        }
        return examFaceVerifyEntity;
    }

    private MultipartEntityBuilder getMultipartEntityBuilder(String bizNo, File basePhotoFile) {
        SysPropertyCacheBean sysProperty = CacheHelper.getSysProperty("app.faceid.notify_url");
        if (!sysProperty.getHasValue()) {
            throw new StatusException("200001", "未找到活体检测回调地址的配置信息");
        }
        String faceidNotifyUrl = sysProperty.getValue().toString();
        //详见:https://faceid.com/pages/documents/5680502
        MultipartEntityBuilder multipartEntityBuilder = MultipartEntityBuilder.create();
        multipartEntityBuilder.addTextBody("api_key", PropertyHolder.getString("$facepp.faceid.api_key"));

        multipartEntityBuilder.addTextBody("api_secret", PropertyHolder.getString("$facepp.faceid.api_secret"));
        multipartEntityBuilder.addTextBody("comparison_type", "0");
        multipartEntityBuilder.addTextBody("return_url", faceidNotifyUrl);
        multipartEntityBuilder.addTextBody("notify_url", faceidNotifyUrl);
        multipartEntityBuilder.addTextBody("biz_no", bizNo);
        multipartEntityBuilder.addTextBody("uuid", bizNo);

        /**
         *
         * screen_replay表示是否开启屏幕翻拍检测。
         *
         * 0：默认值，不开启屏幕翻拍的检测；
         * 1：开启屏幕翻拍检测(若检测屏幕翻拍，返回值将增加屏幕翻拍的结果返回)
         * 注：由于PC端的某些摄像头的驱动，自带增加摄像头视频特效等功能。开启屏幕翻拍检测之后，有可能导致误检增加。
         */
        multipartEntityBuilder.addTextBody("screen_replay","1");

        multipartEntityBuilder.addBinaryBody("image_ref1", basePhotoFile);
        return multipartEntityBuilder;
    }

    /**
     * 获取学生底照文件
     *
     * @param studentId
     * @return
     */
    private File getStudentBasePhotoFile(Long studentId) {
        StudentCacheBean studentBean = CacheHelper.getStudent(studentId);
        String photoUrl = studentBean.getPhotoPath();
        String photoName = systemProperties.getDataDir() + "/" + photoUrl.substring(photoUrl.lastIndexOf("/") + 1);
        photoName = PathUtil.getCanonicalPath(photoName);
        FileDisposeUtil.saveUrlAs(photoUrl, photoName);
        return new File(photoName);
    }

    @Override
    public Integer getFaceLivenessVerifyMinute(Long rootOrgId, Long orgId, Long examId, Long studentId,
                                               Long examRecordDataId, Integer usedMinute) {
        //开启了人脸检测
        if (FaceBiopsyHelper.isFaceVerify(rootOrgId, examId, studentId)) {

            List<ExamFaceLivenessVerifyEntity> faceLivenessVerifys = listFaceVerifyByExamRecordId(examRecordDataId);
            //如果没有进行过人脸检测
            if (faceLivenessVerifys.size() == 0) {

                String faceVerifyStartMinuteStr = ExamCacheTransferHelper.getCachedExamProperty(examId,
                        studentId, ExamProperties.FACE_VERIFY_START_MINUTE.name()).getValue();
                Integer faceVerifyStartMinute = Integer.valueOf(faceVerifyStartMinuteStr);

                String faceVerifyEndMinuteStr = ExamCacheTransferHelper.getCachedExamProperty(examId,
                        studentId, ExamProperties.FACE_VERIFY_END_MINUTE.name()).getValue();
                Integer faceVerifyEndMinute = Integer.valueOf(faceVerifyEndMinuteStr);
                //	case1.如果考生已使用的考试时间(即心跳时间)还未达到系统设置的活体检测开始时间，则实际活体检测时间=random(配置结束时间-配置结束时间)-考试已用时间
                if (usedMinute < faceVerifyStartMinute) {
                    return CommonUtil.calculationRandomNumber(faceVerifyStartMinute, faceVerifyEndMinute) - usedMinute;
                }
                //	case2如果配置开始时间<考生已使用的考试时间<配置结束时间，则实际活体检测时间=random(配置结束时间-考试已用时间)-考试已用时间，如果结果小于1分钟则默认1分钟
                else if (usedMinute >= faceVerifyStartMinute && usedMinute < faceVerifyEndMinute) {
                    int verifyTime = CommonUtil.calculationRandomNumber(usedMinute, faceVerifyEndMinute) - usedMinute;
                    return verifyTime < 1 ? 1 : verifyTime;
                }
                //case3如果考试已用时间>配置结束时间，则默认random(1,4)分钟后开始人脸检测
                else if (usedMinute >= faceVerifyEndMinute) {
                    return CommonUtil.calculationRandomNumber(1, secondFaceCheckMinute);
                }
            } else if (faceLivenessVerifys.size() == 1) {
                //如果已经人脸检测过一次且未成功,再安排一次检测
                ExamFaceLivenessVerifyEntity faceVerify = faceLivenessVerifys.get(0);
                if (faceVerify.getVerifyResult() == null
                        || faceVerify.getVerifyResult() != FaceVerifyResult.VERIFY_SUCCESS) {
                    return CommonUtil.calculationRandomNumber(1, secondFaceCheckMinute);
                }
            }
        }

        return null;
    }
}
