package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.ExamSpecialSettingsType;
import cn.com.qmth.examcloud.api.commons.enums.ExamStageStartExamStatus;
import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.student.api.request.CalcExamScoreReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.CalcFaceBiopsyResultReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.CheckPaperInExamReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordDataIdsReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.UpdateExamRecordDataBatchNumReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.UpdateExamRecordStatusReq;
import cn.com.qmth.examcloud.core.oe.student.api.request.UpdatePartialExamRecordReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.CalcExamScoreResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.CalcFaceBiopsyResultResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.CheckPaperInExamResp;
import cn.com.qmth.examcloud.core.oe.student.base.utils.QuestionTypeUtil;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamFaceLivenessVerifyRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.FaceBiopsyRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamFaceLivenessVerifyEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.FaceBiopsyEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.enums.FaceVerifyResult;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.examwork.api.ExamStageCloudService;
import cn.com.qmth.examcloud.examwork.api.request.ModifyExamStageStartExamStatusReq;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.QuestionAnswerCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.enums.FaceBiopsyScheme;
import cn.com.qmth.examcloud.support.enums.IsSuccess;
import cn.com.qmth.examcloud.support.enums.SyncStatus;
import cn.com.qmth.examcloud.support.examing.ExamQuestion;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.examing.ExamRecordQuestions;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import cn.com.qmth.examcloud.support.helper.FaceBiopsyHelper;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author chenken
 * @date 2018/8/15 11:16
 * @company QMTH
 * @description 考试记录数据服务实现
 */
@Service("examRecordDataService")
public class ExamRecordDataServiceImpl implements ExamRecordDataService {

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private FaceBiopsyRepo faceBiopsyRepo;

    @Autowired
    private ExamFaceLivenessVerifyRepo examFaceLivenessVerifyRepo;

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private ExamStageCloudService examStageCloudService;

    @Transactional
    @Override
    public ExamRecordData createExamRecordData(ExamingSession examingSession, ExamSettingsCacheBean examBean,
                                               CourseCacheBean courseBean, String basePaperId,boolean isFullyObjective,
                                               Long examStageId,Integer examStageOrder) {
        ExamRecordDataEntity examRecordData = new ExamRecordDataEntity();
        examRecordData.setExamId(examBean.getId());
        examRecordData.setExamType(ExamType.valueOf(examBean.getExamType()));

        examRecordData.setExamStudentId(examingSession.getExamStudentId());
        examRecordData.setStudentId(examingSession.getStudentId());
        examRecordData.setOrgId(examingSession.getOrgId());
        examRecordData.setRootOrgId(examingSession.getRootOrgId());

        examRecordData.setCourseId(courseBean.getId());
        examRecordData.setBasePaperId(basePaperId);

        examRecordData.setPaperType(examingSession.getPaperType());

        examRecordData.setEnterExamTime(new Date());
        examRecordData.setIsContinued(false);
        examRecordData.setContinuedCount(0);
        examRecordData.setFaceSuccessCount(0);
        examRecordData.setFaceTotalCount(0);
        examRecordData.setFaceFailedCount(0);
        examRecordData.setFaceStrangerCount(0);
        examRecordData.setExamRecordStatus(ExamRecordStatus.EXAM_ING);
        examRecordData.setSyncStatus(SyncStatus.UNSYNC);
        examRecordData.setIsAllObjectivePaper(isFullyObjective);
        examRecordData.setIsIllegality(false);
        examRecordData.setIsWarn(false);
        examRecordData.setIsAudit(false);
        examRecordData.setExamStageId(examStageId);
        examRecordData.setExamStageOrder(examStageOrder);
        examRecordDataRepo.save(examRecordData);

        //更新场次的开考状态
        if (null != examStageId) {
            if (examBean.getSpecialSettingsEnabled()
                    && ExamSpecialSettingsType.STAGE_BASED == examBean.getSpecialSettingsType()) {
                //更改考务中的考试状态 TODO 高并发下此处可能会影响开考的性能
                ModifyExamStageStartExamStatusReq req = new ModifyExamStageStartExamStatusReq();
                req.setExamStageId(examStageId);
                req.setExamStageStartExamStatus(ExamStageStartExamStatus.STARTED.name());
                examStageCloudService.modifyExamStageStartExamStatus(req);
            }
        }

        ExamRecordData bean = of(examRecordData);
        //存入redis
        saveExamRecordDataCache(examRecordData.getId(), bean);
        return bean;
    }

    @Override
    public void saveExamRecordDataCache(Long examRecordDataId, ExamRecordData data) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        redisClient.set(key, data, 2592000);
    }

    @Override
    public ExamRecordData getExamRecordDataCache(Long examRecordDataId) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        return redisClient.get(key, ExamRecordData.class);
    }

    @Override
    public void deleteExamRecordDataCache(Long examRecordDataId) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        redisClient.delete(key);
    }

    private ExamRecordData of(ExamRecordDataEntity et) {
        ExamRecordData bean = new ExamRecordData();
        bean.setId(et.getId());
        bean.setExamId(et.getExamId());
        bean.setExamType(et.getExamType());

        bean.setExamStudentId(et.getExamStudentId());
        bean.setStudentId(et.getStudentId());
        bean.setCourseId(et.getCourseId());
        bean.setOrgId(et.getOrgId());
        bean.setRootOrgId(et.getRootOrgId());

        bean.setBasePaperId(et.getBasePaperId());

        bean.setPaperType(et.getPaperType());

        bean.setStartTime(et.getStartTime());
        bean.setEndTime(et.getEndTime());
        bean.setUsedExamTime(et.getUsedExamTime());
        bean.setIsContinued(et.getIsContinued());
        bean.setContinuedCount(et.getContinuedCount());
        bean.setIsExceed(et.getIsExceed());
        bean.setFaceSuccessCount(et.getFaceSuccessCount());
        bean.setFaceFailedCount(et.getFaceFailedCount());
        bean.setFaceStrangerCount(et.getFaceStrangerCount());
        bean.setFaceTotalCount(et.getFaceTotalCount());
        bean.setExamRecordStatus(et.getExamRecordStatus());
        bean.setHandInExamType(et.getHandInExamType());
        bean.setFaceVerifyResult(et.getFaceVerifyResult());
        bean.setCleanTime(et.getCleanTime());
        bean.setIsWarn(et.getIsWarn());
        bean.setIsAudit(et.getIsAudit());
        bean.setIsIllegality(et.getIsIllegality());
        bean.setTotalScore(et.getTotalScore());
        bean.setObjectiveScore(et.getObjectiveScore());
        bean.setObjectiveAccuracy(et.getObjectiveAccuracy());
        bean.setSubjectiveScore(et.getSubjectiveScore());
        bean.setSuccPercent(et.getSuccPercent());
        bean.setSyncStatus(et.getSyncStatus());
        bean.setQuestionCount(et.getQuestionCount());
        bean.setIsAllObjectivePaper(et.getIsAllObjectivePaper());
        bean.setEnterExamTime(et.getEnterExamTime());
        bean.setExamStageId(et.getExamStageId());
        bean.setExamStageOrder(et.getExamStageOrder());
        return bean;
    }

    @Override
    public List<Long> getExamRecordDataIds(GetExamRecordDataIdsReq req) {
        Long startId = req.getStartId();
        Long batchNum = req.getBatchNum();
        Integer size = req.getSize();
        if (startId == null) {
            throw new StatusException("1001", "startId 不能为空");
        }
        if (batchNum == null) {
            throw new StatusException("1002", "batchNum 不能为空");
        }
        if (size == null) {
            throw new StatusException("1003", "size 不能为空");
        }
        if (size > 200) {
            throw new StatusException("1004", "size 最大为200");
        }
        List<Long> ids=new ArrayList<Long>();
        List<ExamRecordDataEntity> list = examRecordDataRepo.getLimitExamRecordDataList(batchNum, startId, size);
        if(list!=null&&list.size()!=0) {
        	for(ExamRecordDataEntity e:list) {
    			ids.add(e.getId());
        	}
        }
        return ids;
    }

    @Transactional
    @Override
    public void updateExamRecordDataBatchNum(UpdateExamRecordDataBatchNumReq req) {
        List<Long> ids = req.getIds();
        Long batchNum = req.getBatchNum();
        if (ids == null) {
            throw new StatusException("2001", "ids 不能为空");
        }
        if (batchNum == null) {
            throw new StatusException("2002", "batchNum 不能为空");
        }
        for (Long id : ids) {
            examRecordDataRepo.updateBatchNumById(batchNum, id);
        }
    }

    /**
     * 计算活体检测结果
     *
     * @param req
     * @return
     */
    @Override
    public CalcFaceBiopsyResultResp calcFaceBiopsyResult(CalcFaceBiopsyResultReq req) {
        ExamRecordData examRecordData = this.getExamRecordDataCache(req.getExamRecordDataId());

        //如果活检结果不为空，且活检失败，则判定为违纪有警告
        if (null != examRecordData.getFaceVerifyResult()) {
            if (IsSuccess.FAILED == examRecordData.getFaceVerifyResult()) {
                return new CalcFaceBiopsyResultResp(true, true, IsSuccess.FAILED);
            }
            return new CalcFaceBiopsyResultResp();
        }

        //如果考试记录中的活检记录为空，需要再次计算一次，并更新到考试记录中
        else {
            Long examId = examRecordData.getExamId();
            Long rootOrgId = examRecordData.getRootOrgId();
            Long studentId = examRecordData.getStudentId();

            //是否进行活体检测
            if (FaceBiopsyHelper.isFaceVerify(rootOrgId, examId, studentId)) {
                FaceBiopsyScheme faceBiopsyScheme = FaceBiopsyHelper.getFaceBiopsyScheme(rootOrgId);
                //新活体检测方案
                if (faceBiopsyScheme == FaceBiopsyScheme.NEW) {
                    FaceBiopsyEntity faceBiopsy = faceBiopsyRepo.findByExamRecordDataId(examRecordData.getId());

                    //如果活检结果最终为true,只需要更新活检状态即可
                    if (faceBiopsy != null && faceBiopsy.getResult() != null && faceBiopsy.getResult()) {
                        return new CalcFaceBiopsyResultResp(IsSuccess.SUCCESS);
                    }

                    return new CalcFaceBiopsyResultResp(true, true, IsSuccess.FAILED);
                }
                //旧活体检测方案
                else {
                    List<ExamFaceLivenessVerifyEntity> faceVerifies =
                            examFaceLivenessVerifyRepo.findByExamRecordDataIdOrderById(examRecordData.getId());

                    if (null != faceVerifies && faceVerifies.size() > 0) {
                        ExamFaceLivenessVerifyEntity latestFaceVerify = faceVerifies.get(faceVerifies.size() - 1);
                        //最后一次活检成功，则认为成功，并更新考试记录相关属性
                        if (latestFaceVerify != null && latestFaceVerify.getVerifyResult() != null &&
                                latestFaceVerify.getVerifyResult() == FaceVerifyResult.VERIFY_SUCCESS) {

                            return new CalcFaceBiopsyResultResp(IsSuccess.SUCCESS);
                        }
                    }

                    return new CalcFaceBiopsyResultResp(true, true, IsSuccess.FAILED);
                }
            }
            return new CalcFaceBiopsyResultResp();
        }
    }

    @Override
    public CalcExamScoreResp calcExamScore(CalcExamScoreReq req) {

        Long examRecordDataId = req.getExamRecordDataId();
        ExamRecordQuestions examRecordQuestions = examRecordQuestionsService.getExamRecordQuestions(examRecordDataId);
        // 考生作答记录明细
        List<ExamQuestion> examQuestionList = examRecordQuestions.getExamQuestions();

        double studentObjectiveScoreTotal = 0d;//客观题总分
        int questionSuccessNum = 0;//答题正确数
        int objectiveQuestionsNum = 0;//客观题总数

        //计算客观题总得分和答题正确数
        for (int i = 0; i < examQuestionList.size(); i++) {
            ExamQuestion examQuestion = examQuestionList.get(i);

            if (QuestionTypeUtil.isObjectiveQuestion(examQuestion.getQuestionType())) {
                objectiveQuestionsNum++;

                //如果标准答案为空，则更新标准答案
                if (StringUtils.isEmpty(examQuestion.getCorrectAnswer())) {
                    updateCorrectAnswer(examRecordDataId, examQuestion);
                }

                //如果学生作答正确，则客观分累加，答题数累加
                if (examQuestion.getStudentAnswer() != null
                        && examQuestion.getCorrectAnswer() != null
                        && examQuestion.getStudentAnswer().equals(examQuestion.getCorrectAnswer())) {
                    double questionScore = examQuestion.getQuestionScore().doubleValue();
                    BigDecimal bigDecimalQuestionScore = new BigDecimal(Double.toString(questionScore));
                    BigDecimal bigDecimalObjectiveScoreTotal = new BigDecimal(Double.toString(studentObjectiveScoreTotal));

                    studentObjectiveScoreTotal = bigDecimalQuestionScore.add(bigDecimalObjectiveScoreTotal).doubleValue();
                    questionSuccessNum++;
                }
            }
        }

        //计算客观题答题正确率
        double objectiveAccuracy = 0;
        if (questionSuccessNum > 0 && objectiveQuestionsNum > 0) {
            objectiveAccuracy = Double.valueOf(new DecimalFormat("#.00").format(questionSuccessNum * 100D / objectiveQuestionsNum));
        }

        CalcExamScoreResp resp = new CalcExamScoreResp();
        resp.setObjectiveScore(studentObjectiveScoreTotal);
        resp.setObjectiveAccuracy(objectiveAccuracy);
        resp.setTotalScore(studentObjectiveScoreTotal);//交卷时,总分=客观分得分

        return resp;
    }

    /**
     * 更新部分考试记录数据
     *
     * @param req
     */
    @Transactional
    @Override
    public void updatePartialExamRecord(UpdatePartialExamRecordReq req) {
        if (null == req.getId()) {
            throw new StatusException("300001","考试记录id不允许为空");
        }

        if (StringUtils.isNotEmpty(req.getSyncStatus())) {
            try {
                SyncStatus.valueOf(req.getSyncStatus());
            } catch (IllegalArgumentException e) {
                throw new StatusException("300002","同步状态值不正确");
            }

            examRecordDataRepo.updateExamRecordSyncStatusById(req.getSyncStatus(),req.getId());
        }
    }

    @Override
    public String findCourseNameById(Long id) {
        return examRecordDataRepo.findCourseNameById(id);
    }

    /**
     * 更新客观题答案
     *
     * @param examRecordDataId
     * @param examQuestion
     */
    private void updateCorrectAnswer(Long examRecordDataId, ExamQuestion examQuestion) {
        //更新客观题答案
        QuestionAnswerCacheBean questionAnswerCache = CacheHelper.getQuestionAnswer(examQuestion.getQuestionId());
        List<String> rightAnswerList = questionAnswerCache.getRightAnswers();

        ExamRecordQuestions examRecordQuestions = examRecordQuestionsService.getExamRecordQuestions(examRecordDataId);
        List<ExamQuestion> examQuestionList = examRecordQuestions.getExamQuestions();


        //最小维度的小题单元集合
        List<ExamQuestion> questionUnitList = examQuestionList.stream().
                filter(p -> p.getQuestionId().equals(examQuestion.getQuestionId())).
                sorted((o1, o2) -> o1.getOrder().intValue() - o2.getOrder().intValue()).collect(Collectors.toList());

        //循环保存所有小题单元
        for (int i = 0; i < questionUnitList.size(); i++) {
            ExamQuestion curUnitQuestion = questionUnitList.get(i);
            curUnitQuestion.setCorrectAnswer(rightAnswerList.get(i));
            examRecordQuestionsService.saveExamQuestion(examRecordDataId, curUnitQuestion.getOrder(), curUnitQuestion);

            //更新当前小题的正确答案
            if (curUnitQuestion.getOrder().equals(examQuestion.getOrder())) {
                examQuestion.setCorrectAnswer(rightAnswerList.get(i));
            }
        }

    }

    @Transactional
    @Override
    public void updateExamRecordStatus(UpdateExamRecordStatusReq req) {
        Long id = req.getId();
        String status = req.getExamRecordStatus();
        if (id == null) {
            throw new StatusException("3001", "id 不能为空");
        }
        if (status == null) {
            throw new StatusException("3002", "status 不能为空");
        }
        examRecordDataRepo.updateExamRecordStatusById(status, id);
    }

	@Override
	public CheckPaperInExamResp checkPaperInExam(CheckPaperInExamReq req) {
		Long id=examRecordDataRepo.getRecordIdByPaperId(req.getBasePaperId());
		CheckPaperInExamResp res=new CheckPaperInExamResp();
		if(id==null) {
			res.setInExam(false);
		}else {
			res.setInExam(true);
		}
		return res;
	}
}
