package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.core.oe.student.api.bean.StuExamQuestionBean;
import cn.com.qmth.examcloud.core.oe.student.api.request.GetExamRecordQuestionsReq;
import cn.com.qmth.examcloud.core.oe.student.api.response.GetExamRecordQuestionsResp;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamStudentQuestionInfo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamRecordQuestionTempRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamQuestionTempEntity;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamingSessionService;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultPaper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionStructureWrapper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionUnitWrapper;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionStructure;
import cn.com.qmth.examcloud.question.commons.core.question.DefaultQuestionUnit;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.QuestionCacheBean;
import cn.com.qmth.examcloud.support.examing.*;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

@Service("examRecordQuestionsService")
public class ExamRecordQuestionsServiceImpl implements ExamRecordQuestionsService {

    /**
     * 考生作答内容长度界限，超过此长度的答案内容存入mongo
     */
    private final static int ANWSER_LENGT = 20000;

    // 本地题干缓存，最大200道题，默认缓存120秒
    private Cache<String, String> questionContentCache = CacheBuilder.newBuilder().maximumSize(200)
            .expireAfterWrite(120, TimeUnit.SECONDS).build();

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private ExamRecordQuestionTempRepo examRecordQuestionTempRepo;

    @Autowired
    private ExamingSessionService examingSessionService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Override
    public void saveExamQuestion(Long examRecordDataId, Integer order, ExamQuestion question) {
        String key = RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, order);
        redisClient.set(key, question, 2592000);
    }

    @Override
    public ExamQuestion getExamQuestion(Long examRecordDataId, Integer order) {
        String key = RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, order);
        return redisClient.get(key, ExamQuestion.class);
    }

    @Override
    public void deleteExamQuestion(Long examRecordDataId, Integer order) {
        String key = RedisKeyHelper.getBuilder().studentAnswerKey(examRecordDataId, order);
        redisClient.delete(key);
    }

    @Override
    public ExamRecordQuestions createExamRecordQuestions(Long examRecordDataId, DefaultPaper defaultPaper) {
        ExamRecordQuestions examRecordQuestions = new ExamRecordQuestions();

        List<ExamQuestion> examQuestionEntityList = new ArrayList<ExamQuestion>();
        List<DefaultQuestionGroup> defaultQuestionGroups = defaultPaper.getQuestionGroupList();
        int order = 0;
        for (int i = 0; i < defaultQuestionGroups.size(); i++) {
            DefaultQuestionGroup defaultQuestionGroup = defaultQuestionGroups.get(i);
            List<DefaultQuestionStructureWrapper> defaultQuestionStructureWrappers = defaultQuestionGroup
                    .getQuestionWrapperList();
            for (DefaultQuestionStructureWrapper defaultQuestionStructureWrapper : defaultQuestionStructureWrappers) {
                List<DefaultQuestionUnitWrapper> questionUnitWrapperList = defaultQuestionStructureWrapper
                        .getQuestionUnitWrapperList();
                for (DefaultQuestionUnitWrapper defaultQuestionUnitWrapper : questionUnitWrapperList) {
                    ExamQuestion examQuestionEntity = new ExamQuestion();
                    examQuestionEntity.setExamRecordDataId(examRecordDataId);
                    examQuestionEntity.setIsInMongo(false);
                    examQuestionEntity.setMainNumber(i + 1);
                    examQuestionEntity.setOrder(++order);
                    examQuestionEntity.setQuestionId(defaultQuestionStructureWrapper.getQuestionId());
                    examQuestionEntity.setQuestionScore(defaultQuestionUnitWrapper.getQuestionScore());
                    examQuestionEntity.setQuestionType(defaultQuestionUnitWrapper.getQuestionType());
                    examQuestionEntity.setOptionPermutation(defaultQuestionUnitWrapper.getOptionPermutation());
                    examQuestionEntity.setAudioPlayTimes(null);
                    examQuestionEntity.setAnswerType(defaultQuestionUnitWrapper.getAnswerType());

                    examQuestionEntityList.add(examQuestionEntity);
                }
            }
        }

        examRecordQuestions.setExamQuestions(examQuestionEntityList);
        examRecordQuestions.setExamRecordDataId(examRecordDataId);
        examRecordQuestions.setCreationTime(new Date());
        for (ExamQuestion eq : examRecordQuestions.getExamQuestions()) {
            saveExamQuestion(eq.getExamRecordDataId(), eq.getOrder(), eq);
        }
        return examRecordQuestions;
    }

    @Override
    public ExamRecordQuestions getExamRecordQuestions(Long examRecordDataId) {
        ExamRecordData ed = examRecordDataService.getExamRecordDataCache(examRecordDataId);
        if (ed == null) {
            throw new StatusException("1001", "考试信息未找到");
        }
        Integer quesCount = ed.getQuestionCount();
        ExamRecordQuestions erqs = new ExamRecordQuestions();
        List<ExamQuestion> examQuestions = new ArrayList<ExamQuestion>();
        erqs.setExamQuestions(examQuestions);
        erqs.setExamRecordDataId(examRecordDataId);
        for (int i = 1; i <= quesCount; i++) {
            ExamQuestion eq = getExamQuestion(examRecordDataId, i);
            if (eq.getIsInMongo()) {
                Optional<ExamQuestionTempEntity> op = examRecordQuestionTempRepo.findById(eq.getExamQuestionTempId());
                if (!op.isPresent()) {
                    throw new StatusException("1002", "试题内容未找到");
                }
                copyFromTemp(eq, op.get());
            }
            examQuestions.add(eq);
        }
        return erqs;
    }

    @Override
    public String getQuestionContent(Long studentId, String questionId) {
        ExamingSession examSessionInfo = examingSessionService.getExamingSession(studentId);
        if (examSessionInfo == null
                || examSessionInfo.getExamingStatus().equals(ExamingStatus.INFORMAL)) {
            throw new StatusException("2001", "考试已结束，不允许获取试题内容");
        }

        // 本地缓存key规则：examId_courseCode_paperType_questionId
        String cacheKey = examSessionInfo.getExamId() + "_" + examSessionInfo.getCourseCode() + "_"
                + examSessionInfo.getPaperType() + "_" + questionId;
        String contentJson = questionContentCache.getIfPresent(cacheKey);

        // 如果本地缓存中存在，则从本地缓存中获取
        if (StringUtils.isNotEmpty(contentJson)) {
            return contentJson;
        } else {// 如果本地缓存不存在，则从redis中获取, 并在本地缓存存在存储一份
            QuestionCacheBean getQuestionResp = CacheHelper.getQuestion(examSessionInfo.getExamId(),
                    examSessionInfo.getCourseCode(), examSessionInfo.getPaperType(), questionId);
            DefaultQuestionStructure questionStructure = getQuestionResp.getDefaultQuestion().getMasterVersion();
            List<DefaultQuestionUnit> questionUnits = questionStructure.getQuestionUnitList();

            // 在线考试,清除答案
            if (ExamType.ONLINE.name().equals(examSessionInfo.getExamType())
                    || ExamType.ONLINE_HOMEWORK.name().equals(examSessionInfo.getExamType())) {
                for (DefaultQuestionUnit questionUnit : questionUnits) {
                    questionUnit.setRightAnswer(null);
                }
            }
            String resultJson = JsonUtil.toJson(questionStructure);
            questionContentCache.put(cacheKey, resultJson);
            return resultJson;
        }
    }

    @Override
    public void submitQuestionAnswer(Long studentId, List<ExamStudentQuestionInfo> examQuestionInfos, String referer, String agent) {
        ExamingSession examSessionInfo = examingSessionService.getExamingSession(studentId);
        if (examSessionInfo == null
                || examSessionInfo.getExamingStatus().equals(ExamingStatus.INFORMAL)) {
            throw new StatusException("3001", "考试会话已过期");
        }
        long examRecordDataId = examSessionInfo.getExamRecordDataId();

        if(StringUtils.isEmpty(referer) || StringUtils.isEmpty(agent) || !agent.contains("electron-exam-shell")){
            redisClient.set("OE_SESSION_WARN_"+examSessionInfo.getExamRecordDataId(), true,12 * 60 * 60);
        }

        for (ExamStudentQuestionInfo examQuestionInfo : examQuestionInfos) {
            ExamQuestion eq = getExamQuestion(examRecordDataId, examQuestionInfo.getOrder());
            if (eq == null) {
                throw new StatusException("2002", "试题不存在");
            }
            eq.setStudentAnswer(examQuestionInfo.getStudentAnswer());
            eq.setIsSign(examQuestionInfo.getIsSign());
            eq.setIsAnswer(StringUtils.isNotBlank(eq.getStudentAnswer()));
            eq.setAudioPlayTimes(examQuestionInfo.getAudioPlayTimes());
            //过长存入mongo
            if (eq.getIsAnswer() && eq.getStudentAnswer().length() > ANWSER_LENGT) {
                eq.setIsInMongo(true);
                ExamQuestionTempEntity temp = of(eq);
                examRecordQuestionTempRepo.save(temp);
                eq.setExamQuestionTempId(temp.getId());
                eq.setStudentAnswer(null);
            } else {
                eq.setIsInMongo(false);
            }
            saveExamQuestion(examRecordDataId, examQuestionInfo.getOrder(), eq);
        }
    }

    @Override
    public GetExamRecordQuestionsResp getExamRecordQuestions(GetExamRecordQuestionsReq req) {
        ExamRecordQuestions examRecordQuestions = this.getExamRecordQuestions(req.getExamRecordDataId());

        if (null == examRecordQuestions) {
            throw new StatusException("200002", "找不到该考试的作答记录");
        }

        GetExamRecordQuestionsResp resp = new GetExamRecordQuestionsResp();
        resp.setExamRecordDataId(examRecordQuestions.getExamRecordDataId());
        resp.setCreationTime(examRecordQuestions.getCreationTime());

        List<StuExamQuestionBean> beanList = new ArrayList<>();
        List<ExamQuestion> examQuestions = examRecordQuestions.getExamQuestions();

        for (ExamQuestion examQuestion : examQuestions) {
            StuExamQuestionBean bean = new StuExamQuestionBean();
            bean.setInMongo(examQuestion.getIsInMongo());
            bean.setExamQuestionTempId(examQuestion.getExamQuestionTempId());
            bean.setExamRecordDataId(examQuestion.getExamRecordDataId());
            bean.setMainNumber(examQuestion.getMainNumber());
            bean.setQuestionId(examQuestion.getQuestionId());
            bean.setOrder(examQuestion.getOrder());
            bean.setQuestionScore(examQuestion.getQuestionScore());
            bean.setQuestionType(examQuestion.getQuestionType());
            bean.setCorrectAnswer(examQuestion.getCorrectAnswer());

            //如果作答记录存在mongo中，需要从mongo中重新获取
            if (null != examQuestion.getIsInMongo() && examQuestion.getIsInMongo()) {
                ExamQuestionTempEntity examQuestionTempEntity =
                        GlobalHelper.getEntity(examRecordQuestionTempRepo, examQuestion.getExamQuestionTempId(), ExamQuestionTempEntity.class);
                if (null == examQuestionTempEntity) {
                    throw new StatusException("200003", "找不到试题作答结果");
                }

                bean.setStudentAnswer(examQuestionTempEntity.getStudentAnswer());
            } else {
                bean.setStudentAnswer(examQuestion.getStudentAnswer());
            }

            bean.setStudentScore(examQuestion.getStudentScore());
            bean.setAnswer(examQuestion.getIsAnswer());
            bean.setSign(examQuestion.getIsSign());
            bean.setOptionPermutation(examQuestion.getOptionPermutation());
            bean.setAudioPlayTimes(examQuestion.getAudioPlayTimes());
            bean.setAnswerType(examQuestion.getAnswerType());

            beanList.add(bean);
        }

        resp.setExamQuestions(beanList);

        return resp;
    }

    private ExamQuestionTempEntity of(ExamQuestion eq) {
        ExamQuestionTempEntity tem = new ExamQuestionTempEntity();
        tem.setExamRecordDataId(eq.getExamRecordDataId());
        tem.setMainNumber(eq.getMainNumber());
        tem.setQuestionId(eq.getQuestionId());
        tem.setOrder(eq.getOrder());
        tem.setQuestionScore(eq.getQuestionScore());
        tem.setQuestionType(eq.getQuestionType());
        tem.setCorrectAnswer(eq.getCorrectAnswer());
        tem.setStudentAnswer(eq.getStudentAnswer());
        tem.setStudentScore(eq.getStudentScore());
        tem.setIsAnswer(eq.getIsAnswer());
        tem.setIsSign(eq.getIsSign());
        tem.setOptionPermutation(eq.getOptionPermutation());
        tem.setAudioPlayTimes(eq.getAudioPlayTimes());
        tem.setAnswerType(eq.getAnswerType());
        return tem;
    }

    private void copyFromTemp(ExamQuestion eq, ExamQuestionTempEntity temp) {
        eq.setExamRecordDataId(temp.getExamRecordDataId());
        eq.setMainNumber(temp.getMainNumber());
        eq.setQuestionId(temp.getQuestionId());
        eq.setOrder(temp.getOrder());
        eq.setQuestionScore(temp.getQuestionScore());
        eq.setQuestionType(temp.getQuestionType());
        eq.setCorrectAnswer(temp.getCorrectAnswer());
        eq.setStudentAnswer(temp.getStudentAnswer());
        eq.setStudentScore(temp.getStudentScore());
        eq.setIsAnswer(temp.getIsAnswer());
        eq.setIsSign(temp.getIsSign());
        eq.setOptionPermutation(temp.getOptionPermutation());
        eq.setAudioPlayTimes(temp.getAudioPlayTimes());
        eq.setAnswerType(temp.getAnswerType());
    }
}
